from pathlib import Path

from omegaconf import OmegaConf
import typer

from stairmaze.checker import Checker
from stairmaze.exceptions import CycleDetected, NoRootDag

app = typer.Typer()


@app.command()
def dag(file: Path = typer.Option(...)):
    typer.echo("Checking dag configuration")

    dag = OmegaConf.load(file)

    try:
        Checker.check_dag(dag)
    except NoRootDag as e:
        typer.echo("Failed due to: " + str(e))
        raise typer.Exit(code=1)
    except CycleDetected as e:
        typer.echo("Failed due to: " + str(e))
        raise typer.Exit(code=1)



class DagReducer:
    @classmethod
    def reduce(self, dag):
        nexts = {}
        roots = []
        for task in dag.tasks:
            if "depends_on" in task:
                for dependency in task.depends_on:
                    if dependency not in nexts:
                        nexts[dependency] = []
                    nexts[dependency].append(task.name)
            else:
                roots.append(task.name)

        levels = {root: 1 for root in roots}
        queue = list(roots)
        while len(queue):
            element = queue.pop()
            candidates = nexts.get(element, [])
            for candidate in candidates:
                current = levels.get(candidate, 0)
                levels[candidate] = max(current, levels[element] + 1)
                if current != levels[candidate] and candidate in nexts:
                    queue.append(candidate)

        level2tasks = {}
        for task_name, level in levels.items():
            if level not in level2tasks:
                level2tasks[level] = []
            level2tasks[level].append(task_name)

        levels = [level2tasks[level] for level in sorted(level2tasks)]

        return levels



import json
import os
from pathlib import Path
import shutil

from omegaconf import OmegaConf
import typer

from stairmaze.cli import check
from stairmaze.transformers.dag2sfn_definition import Dag2SFNDefintionTransformer
from stairmaze.transformers.task2job_definition import Task2JobDefinitionTransformer

app = typer.Typer()


@app.command()
def dag(file: Path = typer.Option(...), folder: Path = typer.Option(...)):
    check.dag(file)

    typer.echo("Preparing dag templates (state machine and job definitions)")

    dag = OmegaConf.load(file)

    if os.path.exists(folder):
        shutil.rmtree(folder)
    os.makedirs(folder)

    job_definitions_folder = os.path.join(folder, "batch")
    os.makedirs(job_definitions_folder)

    for task in dag.tasks:
        job_definition = Task2JobDefinitionTransformer.transform(task)

        job_definition_filename = os.path.join(
            job_definitions_folder, f"{task.name}.json"
        )
        with open(job_definition_filename, "w") as fout:
            fout.write(job_definition)

    state_machine = Dag2SFNDefintionTransformer.transform(dag)

    output_filename = os.path.join(folder, "state_machine.json")
    with open(output_filename, "w") as fout:
        fout.write(json.dumps(state_machine, indent=4))



import json
import os

from jinja2 import Environment, FileSystemLoader, select_autoescape

from .tools.dag_reducer import DagReducer


class Dag2SFNDefintionTransformer:
    @classmethod
    def transform(self, dag, dag_reducer=DagReducer):
        template_folder = os.path.join(os.path.dirname(__file__), "templates")
        env = Environment(
            loader=FileSystemLoader(template_folder), autoescape=select_autoescape()
        )

        levels = dag_reducer.reduce(dag)

        state_machine = {"States": {}}
        states = state_machine["States"]
        for level, task_names in enumerate(levels):
            state_name = f"Level_{level}"
            if level == 0:
                state_machine["StartAt"] = state_name
            if len(task_names) > 1:
                state = {"Type": "Parallel", "Branches": []}
                for task_name in task_names:
                    template = env.get_template("task.json.template")
                    rendered = json.loads(
                        template.render(
                            job_definition=f"job-name-{task_name}",
                            job_queue="${job_queue}",
                        )
                    )
                    rendered["End"] = True
                    state["Branches"].append(
                        {"StartAt": task_name, "States": {task_name: rendered}}
                    )
            else:
                task_name = task_names[0]
                template = env.get_template("task.json.template")
                rendered = json.loads(
                    template.render(
                        job_definition=f"job-name-{task_name}", job_queue="${job_queue}"
                    )
                )
                state = rendered
            if level == len(levels) - 1:
                state["End"] = True
            else:
                state["Next"] = f"Level_{level+1}"
            states[state_name] = state
        return state_machine



import os

from jinja2 import Environment, FileSystemLoader, select_autoescape


class Task2JobDefinitionTransformer:
    @classmethod
    def transform(self, task):

        template_folder = os.path.join(os.path.dirname(__file__), "templates")
        env = Environment(
            loader=FileSystemLoader(template_folder), autoescape=select_autoescape()
        )

        if task.type == "docker":
            template = env.get_template("job.json.template")
            rendered = template.render(
                command=list(task.command),
                image=task.image,
                vpcu=task.cpu,
                memory=task.memory,
            )
            return rendered



from stairmaze.exceptions import CycleDetected, NoRootDag


class Checker:
    @classmethod
    def check_dag(self, dag):
        tasks = dag.tasks
        nexts = {}
        roots = []
        for task in tasks:
            if "depends_on" in task:
                for dependency in task.depends_on:
                    if dependency not in nexts:
                        nexts[dependency] = []
                    nexts[dependency].append(task.name)
            else:
                roots.append(task.name)

        if len(roots) == 0:
            raise NoRootDag("Dag without starting nodes")

        ancestors = {task.name: set() for task in tasks}
        queue = [task.name for task in tasks if "depends_on" not in task]
        while len(queue):
            element = queue.pop()
            if element in ancestors[element]:
                raise CycleDetected(f"Task {element} is in a cycle")
            next_nodes = nexts.get(element, [])
            for next_node in next_nodes:
                ancestors[next_node].update(ancestors[element])
                ancestors[next_node].add(element)
                queue.append(next_node)
        return True



import typer

from stairmaze.cli import check, prepare

app = typer.Typer()
app.add_typer(prepare.app, name="prepares")
app.add_typer(check.app, name="checks")

if __name__ == "__main__":  # pragma: no cover
    app()



#!/usr/bin/env python3.8

from distutils.core import setup
import os

import setuptools

packages = setuptools.find_packages(exclude=("tests.*", "tests"))

folder = os.path.dirname(__file__)
with open(os.path.join(folder, "requirements.txt")) as f:
    required = f.read().splitlines()
# required.extend(["setuptools==65.3.0"])

setup(
    name="stairmaze-cli",
    version="0.1.3",
    author="Emilio G. Ortiz García",
    author_email="",
    packages=packages,
    include_package_data=True,
    url="",
    license="LICENSE",
    description="Stairmaze cli",
    long_description=open("README.md").read(),
    install_requires=required,
    entry_points={"console_scripts": ["stairmaze=stairmaze.cli.app:app"]},
)
